
// Recog.java
// Andrew Davison, Jan 2012, as@fivedots.coe.psu.ac.th

/* English recognizer which loads a grammar, prints it,
   and then keeps recognizing user input until crtl-c is typed, or the
   word "exit" is recognized.

   Uses Cloud Garden's TalkingJava implementation of JSAPI
   (http://www.cloudgarden.com/JSAPI/)

   Usage:
      > run Recog hello.gram
      > run Recog numbers.gram
      > run Recog points.gram

*/


import javax.speech.*;
import javax.speech.recognition.*;
import java.io.FileReader;
import java.util.Locale;


public class Recog extends ResultAdapter 
{
  private Recognizer rec;


  public Recog(String fnm)
  {
    try {
      // create a recognizer that supports English
      rec = Central.createRecognizer( new EngineModeDesc(Locale.ENGLISH));
      rec.allocate();   
   
      // load the grammar
      RuleGrammar gram = rec.loadJSGF( new FileReader(fnm) );
      gram.setEnabled(true);
      System.out.println("Loaded grammar from " + fnm);
      printGrammar(gram);

      // add the listener to report results
      rec.addResultListener(this);
  
      // commit the grammar
      rec.commitChanges();
  
      // request focus and start listening
      System.out.println("\nSay something please...");
      rec.requestFocus();
      rec.resume();
    } 
    catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
  }  // end of Recog()


  private void printGrammar(RuleGrammar gram)
  {
   System.out.println("Grammar Name: " + gram.getName());

   String[] rules = gram.listRuleNames();
   for(int i=0; i < rules.length; i++)
     System.out.println("  " + rules[i] + " = " + gram.getRule(rules[i]) + "\n");
  }  // end of printGrammar()



  public void resultAccepted(ResultEvent e) 
  // Process RESULT_ACCEPTED event: print tokens; perhaps exit
  {
    Result r = (Result)(e.getSource());
    ResultToken tokens[] = r.getBestTokens();

    if (tokens == null)
      return;

    // print recognized text
    for (int i = 0; i < tokens.length; i++)
      System.out.print(tokens[i].getSpokenText() + " ");
    System.out.println();

    if (tokens[0].getSpokenText().equals("exit"))
      endRecognizer();
    else
      System.out.println("\nSay something please...");
  }  // end of resultAccepted()



  private void endRecognizer()
  // deallocate the recognizer and exit
  {
    if (rec.testEngineState(Recognizer.ALLOCATED) &&
       !rec.testEngineState(Recognizer.DEALLOCATING_RESOURCES)) {
      try {
        System.out.println("Finishing...\n");
        rec.forceFinalize(true);
        rec.deallocate();
      } 
      catch (Exception ex) {
        ex.printStackTrace();
      }
      System.exit(0);
    }
  }  // end of endRecognizer()



  public void resultRejected(ResultEvent e)
  {
    System.out.println("Sorry, I didn't understand that. Please try again...");
  }  // end of resultRejected()


  // ---------------------------------------------------

  public static void main(String[] args) 
  {
    if (args.length != 1)
      System.out.println("Usage: run Recog <grammar file>");
    else
      new Recog(args[0]);
  }  // end of main()


}  // end of Recog class