
// BORecognizer.java
// Andrew Davison, January 2012, ad@fivedots.coe.psu.ac.th

/* TalkingJava support (http://www.cloudgarden.com/JSAPI/)
   to speed up/slow down the ball, and pause/resume
   and exit the game verbally.   

   The bout.gram contains:
    <cmds> = ( move <speed> ) | pause | resume | exit;
    <speed> = faster | slower;

*/

import javax.speech.*;
import javax.speech.recognition.*;
import java.io.FileReader;
import java.util.Locale;


public class BORecognizer extends ResultAdapter 
{
  private static final String GRAM_FNM = "bout.gram";

  private BreakoutPanel boPan;
  private Recognizer rec;


  public BORecognizer(BreakoutPanel bp)
  {
    boPan = bp;

    try {
      // create a recognizer that supports English
      rec = Central.createRecognizer( new EngineModeDesc(Locale.ENGLISH));
      rec.allocate();   
   
      // load the grammar
      RuleGrammar gram = rec.loadJSGF( new FileReader(GRAM_FNM) );
      gram.setEnabled(true);
      System.out.println("Loaded grammar from " + GRAM_FNM);
      printGrammar(gram);

      // add the listener to report results
      rec.addResultListener(this);
  
      // commit the grammar
      rec.commitChanges();
  
      // request focus and start listening
      System.out.println("\nSay something please...");
      rec.requestFocus();
      rec.resume();
    } 
    catch (Exception e) {
      e.printStackTrace();
      System.exit(1);
    }
  }  // end of BORecognizer()


  private void printGrammar(RuleGrammar gram)
  {
   System.out.println("Grammar Name: " + gram.getName());

   String[] rules = gram.listRuleNames();
   for(int i=0; i < rules.length; i++)
     System.out.println("  " + rules[i] + " = " + gram.getRule(rules[i]) + "\n");
  }  // end of printGrammar()



  public void stop()
  // called from BreakoutPanel: deallocate the recognizer
  {
    if (rec.testEngineState(Recognizer.ALLOCATED) &&
       !rec.testEngineState(Recognizer.DEALLOCATING_RESOURCES)) {
      try {
        System.out.println("Closing down the Speech Recognizer; please wait...\n");
        rec.forceFinalize(true);
        rec.deallocate();
      } 
      catch (Exception ex) {}
    }
  }  // end of stop()



  public void resultAccepted(ResultEvent e) 
  // Receives RESULT_ACCEPTED event: print tokens, process
  {
    Result r = (Result)(e.getSource());
    ResultToken tokens[] = r.getBestTokens();

    // print recognized text
    for (int i = 0; i < tokens.length; i++)
      System.out.print(tokens[i].getSpokenText() + " ");
    System.out.println();

    processCmd(tokens);

    System.out.println("\nSay command please...");
  }  // end of resultAccepted()



  private void processCmd(ResultToken tokens[])
  /* pass commands to the BreakoutPanel    
       <cmds> = ( move <speed> ) | pause | resume | exit;
       <speed> = faster | slower;
  */
  { String cmd = tokens[0].getSpokenText();
    if (cmd.equals("move")) {
      if (tokens.length == 1)
        System.out.println("move has no direction argument");
      else
        boPan.setSpeed(tokens[1].getSpokenText());
    }
    else if (cmd.equals("pause"))
      boPan.speakPause();
    else if (cmd.equals("resume"))
      boPan.speakResume();
    else if (cmd.equals("exit"))
      boPan.speakCloseDown();
    else
      System.out.println("Unknown command: " + cmd);
  }  // end of processCmd()



  public void resultRejected(ResultEvent e)
  {
    System.out.println("Sorry, I didn't understand that. Please try again...");
  }  // end of resultRejected()


}  // end of BORecognizer class